@@ -1,16 +1,29 @@
-/*       +------------------------------------+
- *       | Inspire Internet Relay Chat Daemon |
- *       +------------------------------------+
+/*
+ * InspIRCd -- Internet Relay Chat Daemon
  *
- *  InspIRCd: (C) 2002-2008 InspIRCd Development Team
- * See: http://www.inspircd.org/wiki/index.php/Credits
+ *   Copyright (C) 2012 William Pitcock <nenolod@dereferenced.org>
+ *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
+ *   Copyright (C) 2006, 2009 Robin Burchell <robin+git@viroteck.net>
+ *   Copyright (C) 2007, 2009 Dennis Friis <peavey@inspircd.org>
+ *   Copyright (C) 2008 Thomas Stagner <aquanight@inspircd.org>
+ *   Copyright (C) 2005-2007 Craig Edwards <craigedwards@brainbox.cc>
  *
- * This program is free but copyrighted software; see
- *            the file COPYING for details.
+ * This file is part of InspIRCd.  InspIRCd is free software: you can
+ * redistribute it and/or modify it under the terms of the GNU General Public
+ * License as published by the Free Software Foundation, version 2.
  *
- * ---------------------------------------------------
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ * details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+
+/* $Core */
+
 /*
 dns.cpp - Nonblocking DNS functions.
 Very very loosely based on the firedns library,
@@ -21,7 +34,7 @@ Please do not assume that firedns works like this,
 looks like this, walks like this or tastes like this.
 */
 
-#ifndef WIN32
+#ifndef _WIN32
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <errno.h>
@@ -29,20 +42,14 @@ looks like this, walks like this or tastes like this.
 #include <arpa/inet.h>
 #else
 #include "inspircd_win32wrapper.h"
-#include "inspircd_se_config.h"
 #endif
 
-#include "dns.h"
 #include "inspircd.h"
 #include "socketengine.h"
 #include "configreader.h"
 #include "socket.h"
 
-using irc::sockets::insp_inaddr;
-using irc::sockets::insp_ntoa;
-using irc::sockets::insp_aton;
-using irc::sockets::OpenTCPSocket;
-using irc::sockets::NonBlocking;
+#define DN_COMP_BITMASK	0xC000		/* highest 6 bits in a DN label header */
 
 /** Masks to mask off the responses we get from the DNSRequest methods
  */
@@ -102,38 +109,41 @@ class DNSRequest
 	unsigned long	ttl;		/* Time to live */
 	std::string     orig;		/* Original requested name/ip */
 
-	DNSRequest(InspIRCd* Instance, DNS* dns, int id, const std::string &original);
+	DNSRequest(DNS* dns, int id, const std::string &original);
 	~DNSRequest();
-	DNSInfo ResultIsReady(DNSHeader &h, int length);
+	DNSInfo ResultIsReady(DNSHeader &h, unsigned length);
 	int SendRequests(const DNSHeader *header, const int length, QueryType qt);
 };
 
-class CacheTimer : public InspTimer
+class CacheTimer : public Timer
 {
  private:
-	InspIRCd* ServerInstance;
 	DNS* dns;
  public:
-	CacheTimer(InspIRCd* Instance, DNS* thisdns)
-		: InspTimer(3600, Instance->Time(), true), ServerInstance(Instance), dns(thisdns) { }
+	CacheTimer(DNS* thisdns)
+		: Timer(3600, ServerInstance->Time(), true), dns(thisdns) { }
 
-	virtual void Tick(time_t TIME)
+	virtual void Tick(time_t)
 	{
 		dns->PruneCache();
 	}
 };
 
-class RequestTimeout : public InspTimer
+class RequestTimeout : public Timer
 {
-	InspIRCd* ServerInstance;
 	DNSRequest* watch;
 	int watchid;
  public:
-	RequestTimeout(unsigned long n, InspIRCd* SI, DNSRequest* watching, int id) : InspTimer(n, time(NULL)), ServerInstance(SI), watch(watching), watchid(id)
+	RequestTimeout(unsigned long n, DNSRequest* watching, int id) : Timer(n, ServerInstance->Time()), watch(watching), watchid(id)
 	{
 	}
+	~RequestTimeout()
+	{
+		if (ServerInstance->Res)
+			Tick(0);
+	}
 
-	void Tick(time_t TIME)
+	void Tick(time_t)
 	{
 		if (ServerInstance->Res->requests[watchid] == watch)
 		{
@@ -145,20 +155,33 @@ class RequestTimeout : public InspTimer
 				ServerInstance->Res->Classes[watchid] = NULL;
 			}
 			ServerInstance->Res->requests[watchid] = NULL;
-			DELETE(watch);
-			return;
+			delete watch;
 		}
 	}
 };
 
-/* Allocate the processing buffer */
-DNSRequest::DNSRequest(InspIRCd* Instance, DNS* dns, int id, const std::string &original) : dnsobj(dns)
+CachedQuery::CachedQuery(const std::string &res, unsigned int ttl) : data(res)
 {
-	res = new unsigned char[512];
+	expires = ServerInstance->Time() + ttl;
+}
+
+int CachedQuery::CalcTTLRemaining()
+{
+	int n = expires - ServerInstance->Time();
+	return (n < 0 ? 0 : n);
+}
+
+/* Allocate the processing buffer */
+DNSRequest::DNSRequest(DNS* dns, int rid, const std::string &original) : dnsobj(dns)
+{
+	/* hardening against overflow here:  make our work buffer twice the theoretical
+	 * maximum size so that hostile input doesn't screw us over.
+	 */
+	res = new unsigned char[sizeof(DNSHeader) * 2];
 	*res = 0;
 	orig = original;
-	RequestTimeout* RT = new RequestTimeout(Instance->Config->dns_timeout ? Instance->Config->dns_timeout : 5, Instance, this, id);
-	Instance->Timers->AddTimer(RT); /* The timer manager frees this */
+	RequestTimeout* RT = new RequestTimeout(ServerInstance->Config->dns_timeout ? ServerInstance->Config->dns_timeout : 5, this, rid);
+	ServerInstance->Timers->AddTimer(RT); /* The timer manager frees this */
 }
 
 /* Deallocate the processing buffer */
@@ -211,44 +234,19 @@ inline void DNS::EmptyHeader(unsigned char *output, const DNSHeader *header, con
 /** Send requests we have previously built down the UDP socket */
 int DNSRequest::SendRequests(const DNSHeader *header, const int length, QueryType qt)
 {
+	ServerInstance->Logs->Log("RESOLVER", DEBUG,"DNSRequest::SendRequests");
+
 	unsigned char payload[sizeof(DNSHeader)];
 
 	this->rr_class = 1;
 	this->type = qt;
-		
+
 	DNS::EmptyHeader(payload,header,length);
 
-#ifdef IPV6
-	if (this->dnsobj->socketfamily == AF_INET6)
-	{
-		sockaddr_in6 addr;
-		memset(&addr,0,sizeof(addr));
-		memcpy(&addr.sin6_addr,&dnsobj->myserver6,sizeof(addr.sin6_addr));
-		addr.sin6_family = AF_INET6;
-		addr.sin6_port = htons(DNS::QUERY_PORT);
-		if (sendto(dnsobj->GetFd(), (const char*)payload, length + 12, 0, (sockaddr *) &addr, sizeof(addr)) != length+12)
-			return -1;
-	}
-	else
-	{
-		sockaddr_in addr;
-		memset(&addr,0,sizeof(addr));
-		memcpy(&addr.sin_addr.s_addr,&dnsobj->myserver4,sizeof(addr.sin_addr));
-		addr.sin_family = AF_INET;
-		addr.sin_port = htons(DNS::QUERY_PORT);
-		if (sendto(dnsobj->GetFd(), (const char*)payload, length + 12, 0, (sockaddr *) &addr, sizeof(addr)) != length+12)
-			return -1;
-	}
-#else
-	sockaddr_in addr;
-	memset(&addr,0,sizeof(addr));
-	memcpy(&addr.sin_addr.s_addr, &dnsobj->myserver4, sizeof(addr.sin_addr));
-	addr.sin_family = AF_INET;
-	addr.sin_port = htons(DNS::QUERY_PORT);
-	if (sendto(dnsobj->GetFd(), (const char*)payload, length + 12, 0, (sockaddr *) &addr, sizeof(addr)) != length+12)
+	if (ServerInstance->SE->SendTo(dnsobj, payload, length + 12, 0, &(dnsobj->myserver.sa), sa_size(dnsobj->myserver)) != length+12)
 		return -1;
-#endif
 
+	ServerInstance->Logs->Log("RESOLVER",DEBUG,"Sent OK");
 	return 0;
 }
 
@@ -258,15 +256,33 @@ DNSRequest* DNS::AddQuery(DNSHeader *header, int &id, const char* original)
 	/* Is the DNS connection down? */
 	if (this->GetFd() == -1)
 		return NULL;
-	
+
 	/* Create an id */
-	id = this->PRNG() & DNS::MAX_REQUEST_ID;
+	unsigned int tries = 0;
+	do {
+		id = ServerInstance->GenRandomInt(DNS::MAX_REQUEST_ID);
+		if (++tries == DNS::MAX_REQUEST_ID*5)
+		{
+			// If we couldn't find an empty slot this many times, do a sequential scan as a last
+			// resort. If an empty slot is found that way, go on, otherwise throw an exception
+			id = -1;
+			for (int i = 0; i < DNS::MAX_REQUEST_ID; i++)
+			{
+				if (!requests[i])
+				{
+					id = i;
+					break;
+				}
+			}
 
-	/* If this id is already 'in flight', pick another. */
-	while (requests[id])
-		id = this->PRNG() & DNS::MAX_REQUEST_ID;
+			if (id == -1)
+				throw ModuleException("DNS: All ids are in use");
 
-	DNSRequest* req = new DNSRequest(ServerInstance, this, id, original);
+			break;
+		}
+	} while (requests[id]);
+
+	DNSRequest* req = new DNSRequest(this, id, original);
 
 	header->id[0] = req->id[0] = id >> 8;
 	header->id[1] = req->id[1] = id & 0xFF;
@@ -313,15 +329,11 @@ int DNS::PruneCache()
 
 void DNS::Rehash()
 {
-	ip6munge = false;
-	int portpass = 0;
-
 	if (this->GetFd() > -1)
 	{
-		if (ServerInstance && ServerInstance->SE)
-			ServerInstance->SE->DelFd(this);
-		shutdown(this->GetFd(), 2);
-		close(this->GetFd());
+		ServerInstance->SE->DelFd(this);
+		ServerInstance->SE->Shutdown(this, 2);
+		ServerInstance->SE->Close(this);
 		this->SetFd(-1);
 
 		/* Rehash the cache */
@@ -333,81 +345,56 @@ void DNS::Rehash()
 		this->cache = new dnscache();
 	}
 
-	if ((strstr(ServerInstance->Config->DNSServer,"::ffff:") == (char*)&ServerInstance->Config->DNSServer) ||  (strstr(ServerInstance->Config->DNSServer,"::FFFF:") == (char*)&ServerInstance->Config->DNSServer))
-	{
-		ServerInstance->Log(DEFAULT,"WARNING: Using IPv4 addresses over IPv6 forces some DNS checks to be disabled.");
-		ServerInstance->Log(DEFAULT,"         This should not cause a problem, however it is recommended you migrate");
-		ServerInstance->Log(DEFAULT,"         to a true IPv6 environment.");
-		this->ip6munge = true;
-	}
-
-	this->socketfamily = AF_INET;
-#ifdef IPV6
-	if (strchr(ServerInstance->Config->DNSServer,':'))
-	{
-		this->socketfamily = AF_INET6;
-		inet_pton(AF_INET6, ServerInstance->Config->DNSServer, &this->myserver6);
-	}
-	else
-	{
-		inet_aton(ServerInstance->Config->DNSServer, &this->myserver4);
-		portpass = -1;
-	}
-#else
-	inet_aton(ServerInstance->Config->DNSServer, &this->myserver4);
-#endif
+	irc::sockets::aptosa(ServerInstance->Config->DNSServer, DNS::QUERY_PORT, myserver);
 
 	/* Initialize mastersocket */
-	int s = OpenTCPSocket(ServerInstance->Config->DNSServer, SOCK_DGRAM);
+	int s = socket(myserver.sa.sa_family, SOCK_DGRAM, 0);
 	this->SetFd(s);
-	NonBlocking(s);
 
 	/* Have we got a socket and is it nonblocking? */
 	if (this->GetFd() != -1)
 	{
-		/* Bind the port - port 0 INADDR_ANY */
-		if (!ServerInstance->BindSocket(this->GetFd(), portpass, "", false))
+		ServerInstance->SE->SetReuse(s);
+		ServerInstance->SE->NonBlocking(s);
+		irc::sockets::sockaddrs bindto;
+		memset(&bindto, 0, sizeof(bindto));
+		bindto.sa.sa_family = myserver.sa.sa_family;
+		if (ServerInstance->SE->Bind(this->GetFd(), bindto) < 0)
 		{
 			/* Failed to bind */
-			shutdown(this->GetFd(),2);
-			close(this->GetFd());
+			ServerInstance->Logs->Log("RESOLVER",SPARSE,"Error binding dns socket - hostnames will NOT resolve");
+			ServerInstance->SE->Shutdown(this, 2);
+			ServerInstance->SE->Close(this);
 			this->SetFd(-1);
 		}
-
-		if (this->GetFd() >= 0)
+		else if (!ServerInstance->SE->AddFd(this, FD_WANT_POLL_READ | FD_WANT_NO_WRITE))
 		{
-			/* Hook the descriptor into the socket engine */
-			if (ServerInstance && ServerInstance->SE)
-			{
-				if (!ServerInstance->SE->AddFd(this))
-				{
-					ServerInstance->Log(DEFAULT,"Internal error starting DNS - hostnames will NOT resolve.");
-					shutdown(this->GetFd(),2);
-					close(this->GetFd());
-					this->SetFd(-1);
-				}
-			}
+			ServerInstance->Logs->Log("RESOLVER",SPARSE,"Internal error starting DNS - hostnames will NOT resolve.");
+			ServerInstance->SE->Shutdown(this, 2);
+			ServerInstance->SE->Close(this);
+			this->SetFd(-1);
 		}
 	}
+	else
+	{
+		ServerInstance->Logs->Log("RESOLVER",SPARSE,"Error creating DNS socket - hostnames will NOT resolve");
+	}
 }
 
 /** Initialise the DNS UDP socket so that we can send requests */
-DNS::DNS(InspIRCd* Instance) : ServerInstance(Instance)
+DNS::DNS()
 {
+	ServerInstance->Logs->Log("RESOLVER",DEBUG,"DNS::DNS");
 	/* Clear the Resolver class table */
 	memset(Classes,0,sizeof(Classes));
 
 	/* Clear the requests class table */
 	memset(requests,0,sizeof(requests));
 
-	/* Set the id of the next request to 0
-	 */
-	currid = 0;
-
 	/* DNS::Rehash() sets this to a valid ptr
 	 */
 	this->cache = NULL;
-	
+
 	/* Again, DNS::Rehash() sets this to a
 	 * valid value
 	 */
@@ -417,7 +404,7 @@ DNS::DNS(InspIRCd* Instance) : ServerInstance(Instance)
 	 */
 	this->Rehash();
 
-	this->PruneTimer = new CacheTimer(ServerInstance, this);
+	this->PruneTimer = new CacheTimer(this);
 
 	ServerInstance->Timers->AddTimer(this->PruneTimer);
 }
@@ -465,7 +452,7 @@ int DNS::GetIP(const char *name)
 	DNSHeader h;
 	int id;
 	int length;
-	
+
 	if ((length = this->MakePayload(name, DNS_QUERY_A, 1, (unsigned char*)&h.payload)) == -1)
 		return -1;
 
@@ -513,38 +500,6 @@ int DNS::GetCName(const char *alias)
 	return id;
 }
 
-/** Start lookup of an IP address to a hostname */
-int DNS::GetName(const insp_inaddr *ip)
-{
-	char query[128];
-	DNSHeader h;
-	int id;
-	int length;
-
-#ifdef IPV6
-	unsigned char* c = (unsigned char*)&ip->s6_addr;
-	if (c[0] == 0 && c[1] == 0 && c[2] == 0 && c[3] == 0 &&
-	    c[4] == 0 && c[5] == 0 && c[6] == 0 && c[7] == 0 &&
-	    c[8] == 0 && c[9] == 0 && c[10] == 0xFF && c[11] == 0xFF)
-		sprintf(query,"%d.%d.%d.%d.in-addr.arpa",c[15],c[14],c[13],c[12]);
-	else
-		DNS::MakeIP6Int(query, (in6_addr*)ip);
-#else
-	unsigned char* c = (unsigned char*)&ip->s_addr;
-	sprintf(query,"%d.%d.%d.%d.in-addr.arpa",c[3],c[2],c[1],c[0]);
-#endif
-
-	if ((length = this->MakePayload(query, DNS_QUERY_PTR, 1, (unsigned char*)&h.payload)) == -1)
-		return -1;
-
-	DNSRequest* req = this->AddQuery(&h, id, insp_ntoa(*ip));
-
-	if ((!req) || (req->SendRequests(&h, length, DNS_QUERY_PTR) == -1))
-		return -1;
-
-	return id;
-}
-
 /** Start lookup of an IP address to a hostname */
 int DNS::GetNameForce(const char *ip, ForceProtocol fp)
 {
@@ -552,7 +507,7 @@ int DNS::GetNameForce(const char *ip, ForceProtocol fp)
 	DNSHeader h;
 	int id;
 	int length;
-#ifdef SUPPORT_IP6LINKS
+
 	if (fp == PROTOCOL_IPV6)
 	{
 		in6_addr i;
@@ -561,11 +516,13 @@ int DNS::GetNameForce(const char *ip, ForceProtocol fp)
 			DNS::MakeIP6Int(query, &i);
 		}
 		else
+		{
+			ServerInstance->Logs->Log("RESOLVER",DEBUG,"DNS::GetNameForce IPv6 bad format for '%s'", ip);
 			/* Invalid IP address */
 			return -1;
+		}
 	}
 	else
-#endif
 	{
 		in_addr i;
 		if (inet_aton(ip, &i))
@@ -574,17 +531,33 @@ int DNS::GetNameForce(const char *ip, ForceProtocol fp)
 			sprintf(query,"%d.%d.%d.%d.in-addr.arpa",c[3],c[2],c[1],c[0]);
 		}
 		else
+		{
+			ServerInstance->Logs->Log("RESOLVER",DEBUG,"DNS::GetNameForce IPv4 bad format for '%s'", ip);
 			/* Invalid IP address */
 			return -1;
+		}
 	}
 
-	if ((length = this->MakePayload(query, DNS_QUERY_PTR, 1, (unsigned char*)&h.payload)) == -1)
+	length = this->MakePayload(query, DNS_QUERY_PTR, 1, (unsigned char*)&h.payload);
+	if (length == -1)
+	{
+		ServerInstance->Logs->Log("RESOLVER",DEBUG,"DNS::GetNameForce can't query '%s' using '%s' because it's too long", ip, query);
 		return -1;
+	}
 
 	DNSRequest* req = this->AddQuery(&h, id, ip);
 
-	if ((!req) || (req->SendRequests(&h, length, DNS_QUERY_PTR) == -1))
+	if (!req)
+	{
+		ServerInstance->Logs->Log("RESOLVER",DEBUG,"DNS::GetNameForce can't add query (resolver down?)");
 		return -1;
+	}
+
+	if (req->SendRequests(&h, length, DNS_QUERY_PTR) == -1)
+	{
+		ServerInstance->Logs->Log("RESOLVER",DEBUG,"DNS::GetNameForce can't send (firewall?)");
+		return -1;
+	}
 
 	return id;
 }
@@ -593,7 +566,6 @@ int DNS::GetNameForce(const char *ip, ForceProtocol fp)
  */
 void DNS::MakeIP6Int(char* query, const in6_addr *ip)
 {
-#ifdef SUPPORT_IP6LINKS
 	const char* hex = "0123456789abcdef";
 	for (int index = 31; index >= 0; index--) /* for() loop steps twice per byte */
 	{
@@ -606,9 +578,6 @@ void DNS::MakeIP6Int(char* query, const in6_addr *ip)
 		*query++ = '.'; /* Seperator */
 	}
 	strcpy(query,"ip6.arpa"); /* Suffix the string */
-#else
-	*query = 0;
-#endif
 }
 
 /** Return the next id which is ready, and the result attached to it */
@@ -618,21 +587,17 @@ DNSResult DNS::GetResult()
 	DNSHeader header;
 	DNSRequest *req;
 	unsigned char buffer[sizeof(DNSHeader)];
-	sockaddr* from = new sockaddr[2];
-#ifdef IPV6
-	socklen_t x = this->socketfamily == AF_INET ? sizeof(sockaddr_in) : sizeof(sockaddr_in6);
-#else
-	socklen_t x = sizeof(sockaddr_in);
-#endif
-	const char* ipaddr_from;
-	unsigned short int port_from = 0;
-	int length = _recvfrom(this->GetFd(),(char*)buffer,sizeof(DNSHeader),0,from,&x);
+	irc::sockets::sockaddrs from;
+	memset(&from, 0, sizeof(from));
+	socklen_t x = sizeof(from);
+
+	int length = ServerInstance->SE->RecvFrom(this, (char*)buffer, sizeof(DNSHeader), 0, &from.sa, &x);
 
 	/* Did we get the whole header? */
 	if (length < 12)
 	{
+		ServerInstance->Logs->Log("RESOLVER",DEBUG,"GetResult didn't get a full packet (len=%d)", length);
 		/* Nope - something screwed up. */
-		delete[] from;
 		return DNSResult(-1,"",0,"");
 	}
 
@@ -641,35 +606,17 @@ DNSResult DNS::GetResult()
 	 * is not 53.
 	 * A user could in theory still spoof dns packets anyway
 	 * but this is less trivial than just sending garbage
-	 * to the client, which is possible without this check.
+	 * to the server, which is possible without this check.
 	 *
 	 * -- Thanks jilles for pointing this one out.
 	 */
-#ifdef IPV6
-	char nbuf[MAXBUF];
-	if (this->socketfamily == AF_INET6)
+	if (from != myserver)
 	{
-		ipaddr_from = inet_ntop(AF_INET6, &((sockaddr_in6*)from)->sin6_addr, nbuf, sizeof(nbuf));
-		port_from = ntohs(((sockaddr_in6*)from)->sin6_port);
-	}
-	else
-#endif
-	{
-		ipaddr_from = inet_ntoa(((sockaddr_in*)from)->sin_addr);
-		port_from = ntohs(((sockaddr_in*)from)->sin_port);
-	}
-
-	delete[] from;
-
-	/* We cant perform this security check if you're using 4in6.
-	 * Tough luck to you, choose one or't other!
-	 */
-	if (!ip6munge)
-	{
-		if ((port_from != DNS::QUERY_PORT) || (strcasecmp(ipaddr_from, ServerInstance->Config->DNSServer)))
-		{
-			return DNSResult(-1,"",0,"");
-		}
+		std::string server1 = from.str();
+		std::string server2 = myserver.str();
+		ServerInstance->Logs->Log("RESOLVER",DEBUG,"Got a result from the wrong server! Bad NAT or DNS forging attempt? '%s' != '%s'",
+			server1.c_str(), server2.c_str());
+		return DNSResult(-1,"",0,"");
 	}
 
 	/* Put the read header info into a header class */
@@ -685,6 +632,7 @@ DNSResult DNS::GetResult()
 	if (!requests[this_id])
 	{
 		/* Somehow we got a DNS response for a request we never made... */
+		ServerInstance->Logs->Log("RESOLVER",DEBUG,"Hmm, got a result that we didn't ask for (id=%lx). Ignoring.", this_id);
 		return DNSResult(-1,"",0,"");
 	}
 	else
@@ -764,7 +712,6 @@ DNSResult DNS::GetResult()
 
 			default:
 			break;
-			
 		}
 
 		/* Build the reply with the id and hostname/ip in it */
@@ -775,11 +722,11 @@ DNSResult DNS::GetResult()
 }
 
 /** A result is ready, process it */
-DNSInfo DNSRequest::ResultIsReady(DNSHeader &header, int length)
+DNSInfo DNSRequest::ResultIsReady(DNSHeader &header, unsigned length)
 {
-	int i = 0;
+	unsigned i = 0, o;
 	int q = 0;
-	int curanswer, o;
+	int curanswer;
 	ResourceRecord rr;
  	unsigned short ptr;
 
@@ -787,6 +734,7 @@ DNSInfo DNSRequest::ResultIsReady(DNSHeader &header, int length)
 	rr.type = DNS_QUERY_NONE;
 	rr.rdlength = 0;
 	rr.ttl = 1;	/* GCC is a whiney bastard -- see the XXX below. */
+	rr.rr_class = 0; /* Same for VC++ */
 
 	if (!(header.flags1 & FLAGS_MASK_QR))
 		return std::make_pair((unsigned char*)NULL,"Not a query result");
@@ -841,12 +789,14 @@ DNSInfo DNSRequest::ResultIsReady(DNSHeader &header, int length)
 				else i += header.payload[i] + 1; /* skip length and label */
 			}
 		}
-		if (length - i < 10)
+		if (static_cast<int>(length - i) < 10)
 			return std::make_pair((unsigned char*)NULL,"Incorrectly sized DNS reply");
 
 		/* XXX: We actually initialise 'rr' here including its ttl field */
 		DNS::FillResourceRecord(&rr,&header.payload[i]);
+
 		i += 10;
+		ServerInstance->Logs->Log("RESOLVER",DEBUG,"Resolver: rr.type is %d and this.type is %d rr.class %d this.class %d", rr.type, this->type, rr.rr_class, this->rr_class);
 		if (rr.type != this->type)
 		{
 			curanswer++;
@@ -862,7 +812,7 @@ DNSInfo DNSRequest::ResultIsReady(DNSHeader &header, int length)
 		break;
 	}
 	if ((unsigned int)curanswer == header.ancount)
-		return std::make_pair((unsigned char*)NULL,"No valid answers");
+		return std::make_pair((unsigned char*)NULL,"No A, AAAA or PTR type answers (" + ConvToStr(header.ancount) + " answers)");
 
 	if (i + rr.rdlength > (unsigned int)length)
 		return std::make_pair((unsigned char*)NULL,"Resource record larger than stated");
@@ -874,17 +824,37 @@ DNSInfo DNSRequest::ResultIsReady(DNSHeader &header, int length)
 
 	switch (rr.type)
 	{
+		/*
+		 * CNAME and PTR are compressed.  We need to decompress them.
+		 */
 		case DNS_QUERY_CNAME:
-			/* CNAME and PTR have the same processing code */
 		case DNS_QUERY_PTR:
+		{
+			unsigned short lowest_pos = length;
 			o = 0;
 			q = 0;
 			while (q == 0 && i < length && o + 256 < 1023)
 			{
+				/* DN label found (byte over 63) */
 				if (header.payload[i] > 63)
 				{
 					memcpy(&ptr,&header.payload[i],2);
-					i = ntohs(ptr) - 0xC000 - 12;
+
+					i = ntohs(ptr);
+
+					/* check that highest two bits are set. if not, we've been had */
+					if ((i & DN_COMP_BITMASK) != DN_COMP_BITMASK)
+						return std::make_pair((unsigned char *) NULL, "DN label decompression header is bogus");
+
+					/* mask away the two highest bits. */
+					i &= ~DN_COMP_BITMASK;
+
+					/* and decrease length by 12 bytes. */
+					i -= 12;
+
+					if (i >= lowest_pos)
+						return std::make_pair((unsigned char *) NULL, "Invalid decompression pointer");
+					lowest_pos = i;
 				}
 				else
 				{
@@ -897,37 +867,48 @@ DNSInfo DNSRequest::ResultIsReady(DNSHeader &header, int length)
 						res[o] = 0;
 						if (o != 0)
 							res[o++] = '.';
-						memcpy(&res[o],&header.payload[i + 1],header.payload[i]);
+
+						if (o + header.payload[i] > sizeof(DNSHeader))
+							return std::make_pair((unsigned char *) NULL, "DN label decompression is impossible -- malformed/hostile packet?");
+
+						memcpy(&res[o], &header.payload[i + 1], header.payload[i]);
 						o += header.payload[i];
 						i += header.payload[i] + 1;
 					}
 				}
 			}
 			res[o] = 0;
+		}
 		break;
 		case DNS_QUERY_AAAA:
+			if (rr.rdlength != sizeof(struct in6_addr))
+				return std::make_pair((unsigned char *) NULL, "rr.rdlength is larger than 16 bytes for an ipv6 entry -- malformed/hostile packet?");
+
 			memcpy(res,&header.payload[i],rr.rdlength);
 			res[rr.rdlength] = 0;
 		break;
 		case DNS_QUERY_A:
+			if (rr.rdlength != sizeof(struct in_addr))
+				return std::make_pair((unsigned char *) NULL, "rr.rdlength is larger than 4 bytes for an ipv4 entry -- malformed/hostile packet?");
+
 			memcpy(res,&header.payload[i],rr.rdlength);
 			res[rr.rdlength] = 0;
 		break;
 		default:
-			memcpy(res,&header.payload[i],rr.rdlength);
-			res[rr.rdlength] = 0;
+			return std::make_pair((unsigned char *) NULL, "don't know how to handle undefined type (" + ConvToStr(rr.type) + ") -- rejecting");
 		break;
 	}
-	return std::make_pair(res,"No error");;
+	return std::make_pair(res,"No error");
 }
 
 /** Close the master socket */
 DNS::~DNS()
 {
-	shutdown(this->GetFd(), 2);
-	close(this->GetFd());
+	ServerInstance->SE->Shutdown(this, 2);
+	ServerInstance->SE->Close(this);
 	ServerInstance->Timers->DelTimer(this->PruneTimer);
-	delete this->PruneTimer;
+	if (cache)
+		delete cache;
 }
 
 CachedQuery* DNS::GetCache(const std::string &source)
@@ -938,7 +919,7 @@ CachedQuery* DNS::GetCache(const std::string &source)
 	else
 		return NULL;
 }
-	        
+
 void DNS::DelCache(const std::string &source)
 {
 	cache->erase(source.c_str());
@@ -951,8 +932,9 @@ void Resolver::TriggerCachedResult()
 }
 
 /** High level abstraction of dns used by application at large */
-Resolver::Resolver(InspIRCd* Instance, const std::string &source, QueryType qt, bool &cached, Module* creator) : ServerInstance(Instance), Creator(creator), input(source), querytype(qt)
+Resolver::Resolver(const std::string &source, QueryType qt, bool &cached, Module* creator) : Creator(creator), input(source), querytype(qt)
 {
+	ServerInstance->Logs->Log("RESOLVER",DEBUG,"Resolver::Resolver");
 	cached = false;
 
 	CQ = ServerInstance->Res->GetCache(source);
@@ -970,28 +952,12 @@ Resolver::Resolver(InspIRCd* Instance, const std::string &source, QueryType qt,
 		}
 	}
 
-	insp_inaddr binip;
-
 	switch (querytype)
 	{
 		case DNS_QUERY_A:
 			this->myid = ServerInstance->Res->GetIP(source.c_str());
 		break;
 
-		case DNS_QUERY_PTR:
-			if (insp_aton(source.c_str(), &binip) > 0)
-			{
-				/* Valid ip address */
-				this->myid = ServerInstance->Res->GetName(&binip);
-			}
-			else
-			{
-				this->OnError(RESOLVER_BADIP, "Bad IP address for reverse lookup");
-				throw ModuleException("Resolver: Bad IP address");
-				return;
-			}
-		break;
-
 		case DNS_QUERY_PTR4:
 			querytype = DNS_QUERY_PTR;
 			this->myid = ServerInstance->Res->GetNameForce(source.c_str(), PROTOCOL_IPV4);
@@ -1011,20 +977,22 @@ Resolver::Resolver(InspIRCd* Instance, const std::string &source, QueryType qt,
 		break;
 
 		default:
+			ServerInstance->Logs->Log("RESOLVER",DEBUG,"DNS request with unknown query type %d", querytype);
 			this->myid = -1;
 		break;
 	}
 	if (this->myid == -1)
 	{
-		this->OnError(RESOLVER_NSDOWN, "Nameserver is down");
-		throw ModuleException("Resolver: Couldnt get an id to make a request");
-		/* We shouldnt get here really */
-		return;
+		throw ModuleException("Resolver: Couldn't get an id to make a request");
+	}
+	else
+	{
+		ServerInstance->Logs->Log("RESOLVER",DEBUG,"DNS request id %d", this->myid);
 	}
 }
 
 /** Called when an error occurs */
-void Resolver::OnError(ResolverError e, const std::string &errormessage)
+void Resolver::OnError(ResolverError, const std::string&)
 {
 	/* Nothing in here */
 }
@@ -1047,10 +1015,17 @@ Module* Resolver::GetCreator()
 }
 
 /** Process a socket read event */
-void DNS::HandleEvent(EventType et, int errornum)
+void DNS::HandleEvent(EventType, int)
 {
 	/* Fetch the id and result of the next available packet */
-	DNSResult res = this->GetResult();
+	DNSResult res(0,"",0,"");
+	res.id = 0;
+	ServerInstance->Logs->Log("RESOLVER",DEBUG,"Handle DNS event");
+
+	res = this->GetResult();
+
+	ServerInstance->Logs->Log("RESOLVER",DEBUG,"Result id %d", res.id);
+
 	/* Is there a usable request id? */
 	if (res.id != -1)
 	{
@@ -1068,6 +1043,7 @@ void DNS::HandleEvent(EventType et, int errornum)
 				delete Classes[res.id];
 				Classes[res.id] = NULL;
 			}
+			return;
 		}
 		else
 		{
@@ -1094,6 +1070,7 @@ void DNS::HandleEvent(EventType et, int errornum)
 /** Add a derived Resolver to the working set */
 bool DNS::AddResolverClass(Resolver* r)
 {
+	ServerInstance->Logs->Log("RESOLVER",DEBUG,"AddResolverClass 0x%08lx", (unsigned long)r);
 	/* Check the pointers validity and the id's validity */
 	if ((r) && (r->GetId() > -1))
 	{
@@ -1107,20 +1084,13 @@ bool DNS::AddResolverClass(Resolver* r)
 			Classes[r->GetId()] = r;
 			return true;
 		}
-		else
-			/* Duplicate id */
-			return false;
 	}
-	else
-	{
-		/* Pointer or id not valid.
-		 * Free the item and return
-		 */
-		if (r)
-			delete r;
 
-		return false;
-	}
+	/* Pointer or id not valid, or duplicate id.
+	 * Free the item and return
+	 */
+	delete r;
+	return false;
 }
 
 void DNS::CleanResolvers(Module* module)
@@ -1131,24 +1101,10 @@ void DNS::CleanResolvers(Module* module)
 		{
 			if (Classes[i]->GetCreator() == module)
 			{
-				Classes[i]->OnError(RESLOVER_FORCEUNLOAD, "Parent module is unloading");
+				Classes[i]->OnError(RESOLVER_FORCEUNLOAD, "Parent module is unloading");
 				delete Classes[i];
 				Classes[i] = NULL;
 			}
 		}
 	}
 }
-
-/** Generate pseudo-random number */
-unsigned long DNS::PRNG()
-{
-	unsigned long val = 0;
-	timeval n;
-	serverstats* s = ServerInstance->stats;
-	gettimeofday(&n,NULL);
-	val = (n.tv_usec ^ getpid() ^ geteuid() ^ (this->currid++)) ^ s->statsAccept + n.tv_sec;
-	val = val + s->statsCollisions ^ s->statsDnsGood - s->statsDnsBad;
-	val += (s->statsConnects ^ (unsigned long)s->statsSent ^ (unsigned long)s->statsRecv) - ServerInstance->Config->ports.size();
-	return val;
-}
-
