@@ -36,13 +36,8 @@
 #include "ssl.h"
 
 #ifdef _WIN32
-# pragma comment(lib, "libcrypto.lib")
-# pragma comment(lib, "libssl.lib")
-# pragma comment(lib, "user32.lib")
-# pragma comment(lib, "advapi32.lib")
-# pragma comment(lib, "libgcc.lib")
-# pragma comment(lib, "libmingwex.lib")
-# pragma comment(lib, "gdi32.lib")
+# pragma comment(lib, "ssleay32.lib")
+# pragma comment(lib, "libeay32.lib")
 # undef MAX_DESCRIPTORS
 # define MAX_DESCRIPTORS 10000
 #endif
@@ -249,7 +244,11 @@ class ModuleSSLOpenSSL : public Module
 			ERR_print_errors_cb(error_callback, this);
 		}
 
+#ifdef _WIN32
+		BIO* dhpfile = BIO_new_file(dhfile.c_str(), "r");
+#else
 		FILE* dhpfile = fopen(dhfile.c_str(), "r");
+#endif
 		DH* ret;
 
 		if (dhpfile == NULL)
@@ -259,7 +258,12 @@ class ModuleSSLOpenSSL : public Module
 		}
 		else
 		{
+#ifdef _WIN32
+			ret = PEM_read_bio_DHparams(dhpfile, NULL, NULL, NULL);
+			BIO_free(dhpfile);
+#else
 			ret = PEM_read_DHparams(dhpfile, NULL, NULL, NULL);
+#endif
 			if ((SSL_CTX_set_tmp_dh(ctx, ret) < 0) || (SSL_CTX_set_tmp_dh(clictx, ret) < 0))
 			{
 				ServerInstance->Logs->Log("m_ssl_openssl",DEFAULT, "m_ssl_openssl.so: Couldn't set DH parameters %s. SSL errors follow:", dhfile.c_str());
@@ -268,7 +272,9 @@ class ModuleSSLOpenSSL : public Module
 			DH_free(ret);
 		}
 
+#ifndef _WIN32
 		fclose(dhpfile);
+#endif
 	}
 
 	void On005Numeric(std::string &output)
@@ -633,8 +639,14 @@ class ModuleSSLOpenSSL : public Module
 		char buf[512];
 		X509_NAME_oneline(X509_get_subject_name(cert), buf, sizeof(buf));
 		certinfo->dn = buf;
+		// Make sure there are no chars in the string that we consider invalid
+		if (certinfo->dn.find_first_of("\r\n") != std::string::npos)
+			certinfo->dn.clear();
+
 		X509_NAME_oneline(X509_get_issuer_name(cert), buf, sizeof(buf));
 		certinfo->issuer = buf;
+		if (certinfo->issuer.find_first_of("\r\n") != std::string::npos)
+			certinfo->issuer.clear();
 
 		if (!X509_digest(cert, digest, md, &n))
 		{
