@@ -637,7 +637,7 @@ void OperInfo::init()
 			{
 				this->AllowedUserModes.set();
 			}
-			else if (*c >= 'A' && *c < 'z')
+			else if (*c >= 'A' && *c <= 'z')
 			{
 				this->AllowedUserModes[*c - 'A'] = true;
 			}
@@ -650,7 +650,7 @@ void OperInfo::init()
 			{
 				this->AllowedChanModes.set();
 			}
-			else if (*c >= 'A' && *c < 'z')
+			else if (*c >= 'A' && *c <= 'z')
 			{
 				this->AllowedChanModes[*c - 'A'] = true;
 			}
