@@ -205,6 +205,11 @@ CWin32Exception::CWin32Exception() : exception()
 	dwErrorCode = GetLastError();
 	if( FormatMessageA( FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS, NULL, dwErrorCode, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPSTR)szErrorString, _countof(szErrorString), NULL) == 0 )
 		sprintf_s(szErrorString, _countof(szErrorString), "Error code: %u", dwErrorCode);
+	for (size_t i = 0; i < _countof(szErrorString); i++)
+	{
+		if ((szErrorString[i] == '\r') || (szErrorString[i] == '\n'))
+			szErrorString[i] = 0;
+	}
 }
 
 CWin32Exception::CWin32Exception(const CWin32Exception& other)
